@@ -89,6 +89,9 @@ def _changes_in_subdir(repo, walk_entry, subdir):
     commit = walk_entry.commit
     store = repo.object_store
 
+    if os.path.sep == '\\':
+        subdir = subdir.replace('\\', '/')
+
     parents = walk_entry._get_parents(commit)
 
     if not parents:
@@ -252,6 +255,10 @@ class _ChangeTracker(object):
         self.last_name_by_id = {}
         # Remember uniqueids that have had files deleted.
         self.uniqueids_deleted = set()
+        # Remember files that are changed but not explicitly added so
+        # when we do see an add we can use the more recent tracking
+        # info and if we don't see the add we know to ignore the file.
+        self.seen_but_not_added = {}
 
     def _common(self, uniqueid, sha, version):
         if version not in self.versions:
@@ -284,12 +291,24 @@ class _ChangeTracker(object):
             )
             return
 
-        # A note is being added in this commit. If we have
-        # not seen it before, it was added here and never
-        # changed.
-        if uniqueid not in self.last_name_by_id:
+        if uniqueid in self.seen_but_not_added:
+            # The note was seen for a modify operation already but
+            # this is where it was added. We want to remember where
+            # the modification happened, because that came earlier in
+            # the scan (and therefore later in the history).
+            filename, sha = self.seen_but_not_added[uniqueid]
             self.last_name_by_id[uniqueid] = (filename, sha)
             LOG.info(
+                '%s: copying data for %s from commit %s',
+                uniqueid, filename, sha,
+            )
+            del self.seen_but_not_added[uniqueid]
+        elif uniqueid not in self.last_name_by_id:
+            # The note was added already and we want to keep that
+            # other reference because it came earlier in the scan (and
+            # therefore later in the history).
+            self.last_name_by_id[uniqueid] = (filename, sha)
+            LOG.debug(
                 '%s: new %s in commit %s',
                 uniqueid, filename, sha,
             )
@@ -299,7 +318,7 @@ class _ChangeTracker(object):
                 uniqueid,
             )
 
-    def rename(self, filename, sha, version):
+    def _change(self, filename, sha, version):
         uniqueid = _get_unique_id(filename)
         self._common(uniqueid, sha, version)
 
@@ -313,42 +332,19 @@ class _ChangeTracker(object):
             )
             return
 
+        if uniqueid in self.last_name_by_id:
+            LOG.debug('%s: already added', uniqueid)
+            to_update = self.last_name_by_id
+        else:
+            LOG.debug('%s: seen but not added', uniqueid)
+            to_update = self.seen_but_not_added
+
         # The file is being renamed. We may have seen it
         # before, if there were subsequent modifications,
         # so only store the name information if it is not
         # there already.
-        if uniqueid not in self.last_name_by_id:
-            self.last_name_by_id[uniqueid] = (filename, sha)
-            LOG.info(
-                '%s: update to %s in commit %s',
-                uniqueid, filename, sha,
-            )
-        else:
-            LOG.debug(
-                '%s: renamed file already known with the new name',
-                uniqueid,
-            )
-
-    def modify(self, filename, sha, version):
-        uniqueid = _get_unique_id(filename)
-        self._common(uniqueid, sha, version)
-
-        # If we have recorded that a UID was deleted, that
-        # means that was the last change made to the file and
-        # we can ignore it.
-        if uniqueid in self.uniqueids_deleted:
-            LOG.debug(
-                '%s: has already been deleted, ignoring this change',
-                uniqueid,
-            )
-            return
-
-        # An existing file is being modified. We may have
-        # seen it before, if there were subsequent
-        # modifications, so only store the name
-        # information if it is not there already.
-        if uniqueid not in self.last_name_by_id:
-            self.last_name_by_id[uniqueid] = (filename, sha)
+        if uniqueid not in to_update:
+            to_update[uniqueid] = (filename, sha)
             LOG.info(
                 '%s: update to %s in commit %s',
                 uniqueid, filename, sha,
@@ -359,6 +355,12 @@ class _ChangeTracker(object):
                 uniqueid,
             )
 
+    def rename(self, filename, sha, version):
+        self._change(filename, sha, version)
+
+    def modify(self, filename, sha, version):
+        self._change(filename, sha, version)
+
     def delete(self, filename, sha, version):
         uniqueid = _get_unique_id(filename)
         self._common(uniqueid, sha, version)
@@ -485,6 +487,10 @@ class RenoRepo(repo.Repo):
         commit = self[sha]
         tree = self[commit.tree]
         try:
+            if os.path.sep == '\\':
+                # Dulwich doesn't handle Windows paths, we need to take care of
+                # it ourselves
+                filename = filename.replace('\\', '/')
             mode, blob_sha = tree.lookup_path(self.get_object,
                                               filename.encode('utf-8'))
         except KeyError:
@@ -514,6 +520,11 @@ class Scanner(object):
             self.conf.branch_name_re,
             flags=re.VERBOSE | re.UNICODE,
         )
+        self.branch_name_prefix = self.conf.branch_name_prefix
+        self.closed_branch_tag_re = re.compile(
+            self.conf.closed_branch_tag_re,
+            flags=re.VERBOSE | re.UNICODE,
+        )
         self._ignore_uids = set(
             _get_unique_id(fn)
             for fn in self.conf.ignore_notes
@@ -806,7 +817,7 @@ class Scanner(object):
         "Return true if the file exists at the given commit."
         return bool(self.get_file_at_commit(filename, sha))
 
-    def _get_series_branches(self):
+    def get_series_branches(self):
         "Get branches matching the branch_name_re config option."
         refs = self._repo.get_refs()
         LOG.debug('refs %s', list(refs.keys()))
@@ -819,6 +830,18 @@ class Scanner(object):
             elif r.startswith('refs/heads/'):
                 name = r[11:]
             if name and self.branch_name_re.search(name):
+                LOG.debug('branch name %s', name)
+                branch_names.add(name)
+                continue
+            if not r.startswith('refs/tags/'):
+                continue
+            # See if the ref is a closed branch tag.
+            name = r.rpartition('/')[-1]
+            match = self.closed_branch_tag_re.search(name)
+            if match:
+                name = self.branch_name_prefix + match.group(1)
+                LOG.debug('closed branch tag %s becomes %s',
+                          r.rpartition('/')[-1], name)
                 branch_names.add(name)
         return list(sorted(branch_names))
 
@@ -830,7 +853,7 @@ class Scanner(object):
         if branch.startswith('origin/'):
             branch = branch[7:]
         LOG.debug('looking for the branch before %s', branch)
-        branch_names = self._get_series_branches()
+        branch_names = self.get_series_branches()
         if branch not in branch_names:
             LOG.debug('Could not find branch %r among %s',
                       branch, branch_names)
@@ -894,7 +917,7 @@ class Scanner(object):
                 return candidate
         return None
 
-    def get_notes_by_version(self):
+    def get_notes_by_version(self, branch=None):
         """Return an OrderedDict mapping versions to lists of notes files.
 
         The versions are presented in reverse chronological order.
@@ -902,20 +925,25 @@ class Scanner(object):
         Notes files are associated with the earliest version for which
         they were available, regardless of whether they changed later.
 
-        :param reporoot: Path to the root of the git repository.
-        :type reporoot: str
+        :param branch: The branch to scan. If not provided, using the branch
+            configured in ``self.conf``.
         """
 
         reporoot = self.reporoot
         notesdir = self.conf.notespath
-        branch = self.conf.branch
+        branch = branch or self.conf.branch
         earliest_version = self.conf.earliest_version
         collapse_pre_releases = self.conf.collapse_pre_releases
         stop_at_branch_base = self.conf.stop_at_branch_base
 
-        LOG.info('scanning %s/%s (branch=%s earliest_version=%s)',
-                 reporoot.rstrip('/'), notesdir.lstrip('/'),
-                 branch or '*current*', earliest_version)
+        LOG.info(
+            ('scanning %s/%s '
+             '(branch=%s earliest_version=%s collapse_pre_releases=%s)'),
+            reporoot.rstrip('/'), notesdir.lstrip('/'),
+            branch or '*current*',
+            earliest_version,
+            collapse_pre_releases,
+        )
 
         # Determine the current version, which might be an unreleased or
         # dev version if there are unreleased commits at the head of the
@@ -946,7 +974,7 @@ class Scanner(object):
             # On the current branch, stop at the point where the most
             # recent branch was created, if we can find one.
             LOG.debug('working on current branch without earliest_version')
-            branches = self._get_series_branches()
+            branches = self.get_series_branches()
             if branches:
                 for earlier_branch in reversed(branches):
                     LOG.debug('checking if current branch is later than %s',
@@ -983,6 +1011,7 @@ class Scanner(object):
             # base of the branch, which involves a bit of searching.
             LOG.debug('determining earliest_version from branch')
             branch_base = self._get_branch_base(branch)
+            LOG.debug('branch base %s', branch_base)
             scan_stop_tag = self._find_scan_stop_point(
                 branch_base, versions_by_date,
                 collapse_pre_releases, branch)
@@ -991,6 +1020,8 @@ class Scanner(object):
             else:
                 idx = versions_by_date.index(scan_stop_tag)
                 earliest_version = versions_by_date[idx - 1]
+                LOG.debug('using version before %s as scan stop point',
+                          scan_stop_tag)
             if earliest_version and collapse_pre_releases:
                 if self.pre_release_tag_re.search(earliest_version):
                     # The earliest version won't actually be the pre-release
